 
/*
User interface for Ontology Editing program

Written by Richard Power
Copyright (c) 2013 by Open University

This package is free software; you can redistribute it and/or
modify it under the terms of the GNU Library General Public
License as published by the Free Software Foundation; either
version 3 of the License, or (at your option) any later version.

The package is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Library General Public License for more details.

*/

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.io.*;
import javax.xml.parsers.*;
import org.w3c.dom.*;
import javax.swing.text.*;
import javax.swing.event.CaretEvent;
import javax.swing.event.CaretListener;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import java.util.*;
import java.lang.String;
import javax.swing.border.TitledBorder;

public class Chat extends JFrame
{ private JTextPane messagePane;
  private JTextPane dialoguePane;
  private JTextPane textPane;
  private JTextPane codePane;
  private JTextPane diffPane;
  private JTextPane modelPane;
  private JTextPane entailPane;
  private JScrollPane scrollingMessage;
  private JScrollPane scrollingDialogue;
  private JScrollPane scrollingText;
  private JScrollPane scrollingCode;
  private JScrollPane scrollingDiff;
  private JScrollPane scrollingModel;
  private JScrollPane scrollingEntail;
  private JSplitPane splitPane;
  private JTabbedPane tabbedPane;
  private StyledDocument messageDoc;
  private StyledDocument dialogueDoc;
  private StyledDocument textDoc;
  private StyledDocument codeDoc;
  private StyledDocument diffDoc;
  private StyledDocument modelDoc;
  private StyledDocument entailDoc;
  private MutableAttributeSet standardScript;
  private MutableAttributeSet rejectScript;
  private MutableAttributeSet classScript;
  private MutableAttributeSet propertyScript;
  private MutableAttributeSet individualScript;
  private MutableAttributeSet literalScript;
  private MutableAttributeSet codeScript;
  private MutableAttributeSet responseScript;
  public static final int FONT_SIZE = 18;
  public static final int CODE_FONT_SIZE = 16;
  public static final String FONT_FAMILY = "Arial";
  public static final String CODE_FONT_FAMILY = "Courier";
  public static final boolean BOLD_FONT = false;
  private Sentence sentence;
  private ArrayList<Sentence> sentences;
  private int dialogueLength;
  private int textLength;
  private int codeLength;
  private String codeString;
  private Model currentModel;
  private Model lastModel;
  private Word currentWord;

  private Action saveAction = new SaveAction();
  private Action openAction = new OpenAction();
  private Action demoAction = new DemoAction();

  private static final String demoFileName = "demo.txt";
  private static final String DEFAULT_PATH = "./";

  private static final String codeFrame = "Prefix(:=<http://www.example.com/ont#>)\nOntology( <http://www.example.com/ont>\n)";

  public static void main(String[] args)
  { System.setProperty("apple.laf.useScreenMenuBar", "true");
    Chat editor = new Chat();
    editor.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
    editor.setLocation(200,200);
    editor.setVisible(true);
  }

  public Chat()
  { super("Dialogue-Based Ontology Editor");
    messagePane = new JTextPane();
    dialoguePane = new JTextPane();
    textPane = new JTextPane();
    codePane = new JTextPane();
    diffPane = new JTextPane();
    modelPane = new JTextPane();
    entailPane = new JTextPane();

    messagePane.setBorder(BorderFactory.createTitledBorder(
      BorderFactory.createCompoundBorder(
	BorderFactory.createLoweredBevelBorder(), 
        BorderFactory.createEmptyBorder(10, 10, 10, 10)), 
      "Message",
      TitledBorder.LEADING,
      TitledBorder.TOP,
      new Font(FONT_FAMILY, Font.BOLD, FONT_SIZE)));

    dialoguePane.setBorder(BorderFactory.createTitledBorder(
      BorderFactory.createCompoundBorder(
	 BorderFactory.createLoweredBevelBorder(), 
         BorderFactory.createEmptyBorder(10,10,10,10)), 
      "Dialogue",
      TitledBorder.LEADING,
      TitledBorder.TOP,
      new Font(FONT_FAMILY,Font.BOLD,FONT_SIZE)));

    textPane.setBorder(BorderFactory.createTitledBorder(
      BorderFactory.createCompoundBorder(
	 BorderFactory.createLoweredBevelBorder(), 
         BorderFactory.createEmptyBorder(10,10,10,10)), 
      "Axioms",
      TitledBorder.LEADING,
      TitledBorder.TOP,
      new Font(FONT_FAMILY,Font.BOLD,FONT_SIZE)));

    codePane.setBorder(BorderFactory.createTitledBorder(
      BorderFactory.createCompoundBorder(
	 BorderFactory.createLoweredBevelBorder(), 
         BorderFactory.createEmptyBorder(10,10,10,10)), 
      "Code",
      TitledBorder.LEADING,
      TitledBorder.TOP,
      new Font(FONT_FAMILY,Font.BOLD,FONT_SIZE)));

    diffPane.setBorder(BorderFactory.createTitledBorder(
      BorderFactory.createCompoundBorder(
	 BorderFactory.createLoweredBevelBorder(), 
         BorderFactory.createEmptyBorder(10,10,10,10)), 
      "Glossary",
      TitledBorder.LEADING,
      TitledBorder.TOP,
      new Font(FONT_FAMILY,Font.BOLD,FONT_SIZE)));

    modelPane.setBorder(BorderFactory.createTitledBorder(
      BorderFactory.createCompoundBorder(
	 BorderFactory.createLoweredBevelBorder(), 
         BorderFactory.createEmptyBorder(10,10,10,10)), 
      "Model",
      TitledBorder.LEADING,
      TitledBorder.TOP,
      new Font(FONT_FAMILY,Font.BOLD,FONT_SIZE)));

    entailPane.setBorder(BorderFactory.createTitledBorder(
      BorderFactory.createCompoundBorder(
	 BorderFactory.createLoweredBevelBorder(), 
         BorderFactory.createEmptyBorder(10,10,10,10)), 
      "Entailments",
      TitledBorder.LEADING,
      TitledBorder.TOP,
      new Font(FONT_FAMILY,Font.BOLD,FONT_SIZE)));

    messageDoc = messagePane.getStyledDocument();
    dialogueDoc = dialoguePane.getStyledDocument();
    textDoc = textPane.getStyledDocument();
    codeDoc = codePane.getStyledDocument();
    diffDoc = diffPane.getStyledDocument();
    modelDoc = modelPane.getStyledDocument();
    entailDoc = entailPane.getStyledDocument();

    standardScript = new SimpleAttributeSet();
    StyleConstants.setForeground(standardScript, Color.black);
    StyleConstants.setAlignment(standardScript, StyleConstants.ALIGN_LEFT);
    StyleConstants.setFontSize(standardScript, FONT_SIZE);
    StyleConstants.setFontFamily(standardScript, FONT_FAMILY);
    StyleConstants.setBold(standardScript, BOLD_FONT);
    StyleConstants.setLineSpacing(standardScript, 0.01f);
    StyleConstants.setSpaceBelow(standardScript, 5);

    responseScript = new SimpleAttributeSet();
    StyleConstants.setForeground(responseScript, Color.black);
    StyleConstants.setAlignment(responseScript, StyleConstants.ALIGN_LEFT);
    StyleConstants.setFontSize(responseScript, FONT_SIZE);
    StyleConstants.setFontFamily(responseScript, FONT_FAMILY);
    StyleConstants.setBold(responseScript, BOLD_FONT);
    StyleConstants.setItalic(responseScript, true);
    StyleConstants.setLineSpacing(responseScript, 0.01f);
    StyleConstants.setSpaceBelow(responseScript, 5);

    codeScript = new SimpleAttributeSet();
    StyleConstants.setForeground(codeScript, Color.black);
    StyleConstants.setAlignment(codeScript, StyleConstants.ALIGN_LEFT);
    StyleConstants.setFontSize(codeScript, CODE_FONT_SIZE);
    StyleConstants.setFontFamily(codeScript, CODE_FONT_FAMILY);
    StyleConstants.setBold(codeScript, BOLD_FONT);
    StyleConstants.setLineSpacing(codeScript, 0.01f);
    StyleConstants.setSpaceBelow(codeScript, 5);

    messageDoc.setParagraphAttributes(0, 0, standardScript, true);
    dialogueDoc.setParagraphAttributes(0, 0, standardScript, true);
    textDoc.setParagraphAttributes(0, 0, standardScript, true);
    codeDoc.setParagraphAttributes(0, 0, codeScript, true);
    diffDoc.setParagraphAttributes(0, 0, standardScript, true);
    modelDoc.setParagraphAttributes(0, 0, standardScript, true);
    entailDoc.setParagraphAttributes(0, 0, standardScript, true);

    rejectScript = new SimpleAttributeSet();
    StyleConstants.setForeground(rejectScript, Color.red);
    StyleConstants.setFontSize(rejectScript, FONT_SIZE);
    StyleConstants.setBold(rejectScript, BOLD_FONT);
    StyleConstants.setFontFamily(rejectScript, FONT_FAMILY);

    classScript = new SimpleAttributeSet();
    StyleConstants.setForeground(classScript, new Color(205,133,0));
    StyleConstants.setFontSize(classScript, FONT_SIZE);
    StyleConstants.setBold(classScript, BOLD_FONT);
    StyleConstants.setFontFamily(classScript, FONT_FAMILY);

    propertyScript = new SimpleAttributeSet();
    StyleConstants.setForeground(propertyScript, Color.blue);
    StyleConstants.setFontSize(propertyScript, FONT_SIZE);
    StyleConstants.setBold(propertyScript, BOLD_FONT);
    StyleConstants.setFontFamily(propertyScript, FONT_FAMILY);

    individualScript = new SimpleAttributeSet();
    StyleConstants.setForeground(individualScript, Color.magenta);
    StyleConstants.setFontSize(individualScript, FONT_SIZE);
    StyleConstants.setBold(individualScript, BOLD_FONT);
    StyleConstants.setFontFamily(individualScript, FONT_FAMILY);

    literalScript = new SimpleAttributeSet();
    StyleConstants.setForeground(literalScript, new Color(0,170,110));
    StyleConstants.setFontSize(literalScript, FONT_SIZE);
    StyleConstants.setBold(literalScript, BOLD_FONT);
    StyleConstants.setFontFamily(literalScript, FONT_FAMILY);

    scrollingMessage = new JScrollPane(messagePane);
    scrollingDialogue = new JScrollPane(dialoguePane);
    scrollingText = new JScrollPane(textPane);
    scrollingCode = new JScrollPane(codePane);
    scrollingDiff = new JScrollPane(diffPane);
    scrollingModel = new JScrollPane(modelPane);
    scrollingEntail = new JScrollPane(entailPane);
 
    dialoguePane.setEditable(false);
    textPane.setEditable(false);
    codePane.setEditable(false);
    diffPane.setEditable(false);
    modelPane.setEditable(false);
    entailPane.setEditable(false);

    splitPane = new JSplitPane(JSplitPane.VERTICAL_SPLIT,
                               scrollingDialogue, scrollingMessage);
    splitPane.setContinuousLayout(true);
    splitPane.setDividerSize(2);
    Dimension minimumSize = new Dimension(100, 80);
    Dimension preferredSize = new Dimension(100, 800);
    scrollingDialogue.setMinimumSize(minimumSize);
    scrollingDialogue.setPreferredSize(preferredSize);
    scrollingMessage.setMinimumSize(minimumSize);
    scrollingMessage.setPreferredSize(minimumSize);

    tabbedPane = new JTabbedPane();
    tabbedPane.addTab("Dialogue", splitPane);
    tabbedPane.addTab("Axioms", scrollingText);
    tabbedPane.addTab("Entailments", scrollingEntail);
    tabbedPane.addTab("Code", scrollingCode);
    tabbedPane.addTab("Model", scrollingModel);
    tabbedPane.addTab("Glossary", scrollingDiff);

    Container content = getContentPane();
    content.add(tabbedPane, BorderLayout.CENTER);
    setJMenuBar(createMenuBar());    

    sentences = new ArrayList<Sentence>();

    currentModel = new Model(sentences);
    lastModel = new Model(sentences);

    textPane.setText("");
    codePane.setText(codeFrame);
    diffPane.setText("");
    modelPane.setText("");
    entailPane.setText("");

    CaretListener editListener = new CaretListener()
    { public void caretUpdate(CaretEvent caretEvent)
      { String message = messagePane.getText();

        int ascii = 0;
        int mlength = message.length();
        if (mlength>0) ascii = message.codePointAt(mlength-1);

        if ((ascii==10)||(ascii==13))
        { message = message.replace("\n","");
          message = message.replace("\r","");
        }

        sentence = new Sentence(message);
        codeString = sentence.getOwlFunctionalSyntax();
        currentWord = sentence.getCurrentWord(mlength-1);

        dialogueLength = dialogueDoc.getLength();
        textLength = textDoc.getLength();

        if ((ascii==10) && (sentence.sentenceIsCorrect())) 
        { if (sentence.getSpeechAct()==Sentence.RETRACT) updatePanesRetract();
          else if (sentence.getSpeechAct()==Sentence.CONFIRM) updatePanesConfirm();
          else if (sentence.getSpeechAct()==Sentence.EXPLAIN) updatePanesExplain();
          else if (sentence.getSpeechAct()==Sentence.RETRIEVE) updatePanesRetrieve();
          else if (sentence.getSpeechAct()==Sentence.DESCRIBE) updatePanesDescribe();
          else updatePanesAssert(false);
          dialoguePane.setCaretPosition(dialogueDoc.getLength());
        }
        else 
        { SwingUtilities.invokeLater(new Runnable()
          { public void run()
            { recolourSentence(messageDoc,0); } });
        }
      }
    };

    messagePane.addCaretListener(editListener);
    setSize(800, 1000);
  }

  private void updatePanesAssert(boolean loading)
  { String report = "";
    String update = "";
    String smodel = "";
    String entail = "";
    if (repeatedAssertion()==true) report = "Already asserted.";
    else
    { Model updatedModel = new Model(sentences,sentence);
      sentences.add(sentence);
      lastModel = currentModel;
      currentModel = new Model(sentences);
      smodel = currentModel.report();
      ModelDiff diff = new ModelDiff(currentModel,updatedModel);
      report = diff.responseFull();
//      update = diff.reportNL();
      entail = currentModel.reportEntailments();
      if (report.equals("Already inferred.")) sentences.remove(sentence);
      else
      { addText(textDoc,sentence.getTextString()+"\n",standardScript);
        addCodeText(codeString+"\n");
      }
      Glossary glossary = new Glossary(currentModel);
      update = glossary.getText();
      modelPane.setText(smodel);
    }
    addText(dialogueDoc,sentence.getTextString()+"\n",standardScript);
    addText(dialogueDoc,report+"\n\n",responseScript);
    diffPane.setText(update);
    entailPane.setText(entail);
    if (loading==true) recolourSentence(dialogueDoc,dialogueLength);
    else SwingUtilities.invokeLater(new Runnable()
      { public void run() { recolourSentence(dialogueDoc,dialogueLength); } });
    if (loading==true) recolourSentence(textDoc,textLength);
    else SwingUtilities.invokeLater(new Runnable()
      { public void run() { recolourSentence(textDoc,textLength); } });
    if (loading==true) messagePane.setText("");
    else SwingUtilities.invokeLater(new Runnable()
      { public void run() { messagePane.setText(""); } });
  }

  private void updatePanesRetract()
  { String report = "";
    String update = "";
    String smodel = "";
    String entail = "";
    if (repeatedAssertion()==false) report = "Never asserted.";
    else
    { Sentence retracted = sentence;
      for (Sentence sent : sentences)
      { if (sent.getOwlFunctionalSyntax().equals(sentence.getOwlFunctionalSyntax()))
        retracted = sent;
      }
      sentences.remove(retracted);
      lastModel = currentModel;
      currentModel = new Model(sentences);
      ModelDiff diff = new ModelDiff(currentModel,lastModel);
      report = diff.retractionResponseFull();
//      update = diff.reportNL();
      smodel = currentModel.report();
      entail = currentModel.reportEntailments();
      removeDocumentString(textDoc,retracted.getTextString());
      removeDocumentString(codeDoc,retracted.getOwlFunctionalSyntax());
      modelPane.setText(smodel);
      Glossary glossary = new Glossary(currentModel);
      update = glossary.getText();
      modelPane.setText(smodel);
    }
    addText(dialogueDoc,sentence.getTextString()+"\n",standardScript);
    addText(dialogueDoc,report+"\n\n",responseScript);
    diffPane.setText(update);
    entailPane.setText(entail);
    SwingUtilities.invokeLater(new Runnable()
      { public void run() { recolourSentence(dialogueDoc,dialogueLength); } });
    SwingUtilities.invokeLater(new Runnable()
      { public void run() { recolourSentence(textDoc,textLength); } });
    SwingUtilities.invokeLater(new Runnable()
      { public void run() { messagePane.setText(""); } });
  }

  private void updatePanesConfirm()
  { String report = "";
    String update = "";
    String smodel = "";
    String entail = "";
    if (repeatedAssertion()==true) report = "Yes, asserted.";
    else
    { Model updatedModel = new Model(sentences,sentence);
      sentences.add(sentence);
      lastModel = currentModel;
      currentModel = new Model(sentences);
      ModelDiff diff = new ModelDiff(currentModel,updatedModel);
      report = diff.response();
      if (report.equals("Already inferred.")) report = "Yes, inferred.";
      else report = "No, unconfirmed.";
      sentences.remove(sentence);
    }
    addText(dialogueDoc,sentence.getTextString()+"\n",standardScript);
    addText(dialogueDoc,report+"\n\n",responseScript);
    SwingUtilities.invokeLater(new Runnable()
      { public void run() { recolourSentence(dialogueDoc,dialogueLength); } });
    SwingUtilities.invokeLater(new Runnable()
      { public void run() { messagePane.setText(""); } });
  }

  private void updatePanesRetrieve()
  { String report = "";
    String update = "";
    String smodel = "";
    String entail = "";

    Model updatedModel = new Model(sentences,sentence);
    lastModel = currentModel;
    report = updatedModel.retrievalResponse();

    addText(dialogueDoc,sentence.getTextString()+"\n",standardScript);
    addText(dialogueDoc,report+"\n\n",responseScript);
    SwingUtilities.invokeLater(new Runnable()
      { public void run() { recolourSentence(dialogueDoc,dialogueLength); } });
    SwingUtilities.invokeLater(new Runnable()
      { public void run() { messagePane.setText(""); } });
  }

  private void updatePanesDescribe()
  { String report = "";
    String update = "";
    String smodel = "";
    String entail = "";

    Model updatedModel = new Model(sentences,sentence);
    lastModel = currentModel;
    report = updatedModel.descriptionResponse();

    addText(dialogueDoc,sentence.getTextString()+"\n",standardScript);
    addText(dialogueDoc,report+"\n\n",responseScript);
    SwingUtilities.invokeLater(new Runnable()
      { public void run() { recolourSentence(dialogueDoc,dialogueLength); } });
    SwingUtilities.invokeLater(new Runnable()
      { public void run() { messagePane.setText(""); } });
  }

  private void updatePanesExplain()
  { String report = "";
    String update = "";
    String smodel = "";
    String entail = "";
    if (repeatedAssertion()==true) report = "Asserted.";
    else
    { Model updatedModel = new Model(sentences,sentence);
      sentences.add(sentence);
      lastModel = currentModel;
      currentModel = new Model(sentences);
      ModelDiff diff = new ModelDiff(currentModel,updatedModel);
      report = diff.response();
      if (report.equals("Already inferred.")) report = updatedModel.proofResponse();
      else report = "Unconfirmed.\n";
      sentences.remove(sentence);
    }
    addText(dialogueDoc,sentence.getTextString()+"\n",standardScript);
    addText(dialogueDoc,report+"\n",responseScript);
    SwingUtilities.invokeLater(new Runnable()
      { public void run() { recolourSentence(dialogueDoc,dialogueLength); } });
    SwingUtilities.invokeLater(new Runnable()
      { public void run() { messagePane.setText(""); } });
  }

  private void removeDocumentString(StyledDocument doc, String s)
  { ArrayList<Integer> spans = getSpans(doc);
    for (int i=0; i<spans.size()-1; i++)
    { int start = 0;
      if (i>0) start = spans.get(i)+1;
      int len = spans.get(i+1)-start;
      try
      { String ss = doc.getText(start,len);
        if (ss.equals(s)) { doc.remove(start,len+1); return; }
      }
      catch (BadLocationException badLocationException)
      { System.out.println("Out of range"); }
    }
  }

  private ArrayList<Integer> getSpans(StyledDocument doc)
  { ArrayList<Integer> spans = new ArrayList<Integer>();
    spans.add(0);
    for (int i=0; i<doc.getLength(); i++)
    try
    { if (doc.getText(i,1).equals("\n")) spans.add(i); }
    catch (BadLocationException badLocationException)
    { System.err.println("Document update error"); }
    return spans;
  }

// Give starting place of the sentence in the whole string
// Also specify the document to be recoloured
  private void recolourSentence(StyledDocument doc, int pos)
  { Word[] words = sentence.getWords();
    int slen = words.length;
    for (int i=0; i<slen; i++)
    { if (words[i].equals(currentWord) && doc.equals(messageDoc))
      recolourWord(doc,pos,words[i],standardScript);
      else if (words[i].getWordRole() == Sentence.ERROR)
      recolourWord(doc,pos,words[i],rejectScript);
      else if (words[i].getWordRole() == Sentence.INDIVIDUAL)
      recolourWord(doc,pos,words[i],individualScript);
      else if (words[i].getWordRole() == Sentence.LITERAL)
      recolourWord(doc,pos,words[i],literalScript);
      else if (words[i].getWordRole() == Sentence.CLASS)
      recolourWord(doc,pos,words[i],classScript);
      else if (words[i].getWordRole() == Sentence.PROPERTY)
      recolourWord(doc,pos,words[i],propertyScript);
      else
      recolourWord(doc,pos,words[i],standardScript);
    }
  }

  private void recolourWord(StyledDocument doc, int pos,
                            Word word, AttributeSet aset)
  { int wordStart = word.getWordStart() + pos;
    int wordLength = word.getWordFinish()-word.getWordStart();
    doc.setCharacterAttributes(wordStart,wordLength,aset,false);
  }

  private void addText(StyledDocument doc, String text, AttributeSet aset)
  { try { doc.insertString(doc.getLength(),text,aset); }
    catch (BadLocationException badLocationException)
    { System.err.println("Document update error"); }
  }

  private void addCodeText(String text)
  { try { codeDoc.insertString(codeDoc.getLength()-1,text,codeScript); }
    catch (BadLocationException badLocationException)
    { System.err.println("Code update error"); }
  }

  private boolean repeatedAssertion()
  { boolean repeated = false;
    for (Sentence sent : sentences)
    if (sent.getOwlFunctionalSyntax().equals(sentence.getOwlFunctionalSyntax()))
    repeated = true;
    return repeated;
  }

  private JMenuBar createMenuBar()
  { JMenuBar menubar = new JMenuBar();
    JMenu file = new JMenu("File");
    menubar.add(file);
    file.add(saveAction);
    file.add(openAction);
    file.add(demoAction);
    return menubar;
  }

  class SaveAction extends AbstractAction
  { public SaveAction() { super("Save"); }
    public void actionPerformed(ActionEvent ev)
    { JFileChooser chooser = new JFileChooser(DEFAULT_PATH);
      if (chooser.showSaveDialog(Chat.this) != JFileChooser.APPROVE_OPTION) return;
      File file = chooser.getSelectedFile();
      if (file == null) return;
      FileWriter writer = null;
      try
      { writer = new FileWriter(file);
        textPane.write(writer);
      }
      catch (IOException ex)
      { System.out.println("File Not Saved");
      }
      finally
      { if (writer != null)
        { try { writer.close(); }
          catch (IOException x) { }
        }
      }
    }
  }

  class OpenAction extends AbstractAction
  { public OpenAction() { super("Open"); }
    public void actionPerformed(ActionEvent ev)
    { JFileChooser chooser = new JFileChooser(DEFAULT_PATH);
      if (chooser.showOpenDialog(Chat.this) != JFileChooser.APPROVE_OPTION) return;
      File file = chooser.getSelectedFile();
//      System.out.println(file.getAbsolutePath());
      if (file == null) return;
      try
      { String tmp;
        String textData = null;
        BufferedReader in = new BufferedReader(new FileReader(file));
        tmp = in.readLine();
        textData = tmp;
        while (tmp != null)
        { tmp = in.readLine();
          textData = textData + "\n" + tmp;
        }
        textData = textData.substring(0,textData.length()-5);
        loadText(textData);
        in.close();
      }
      catch (IOException ex) { System.out.println("File Not Found"); }
    }
  }

  class DemoAction extends AbstractAction
  { public DemoAction() { super("Demo"); }
    public void actionPerformed(ActionEvent ev)
    { try
      { String tmp;
        String textData = null;
        BufferedReader in = new BufferedReader(new FileReader(demoFileName));
        tmp = in.readLine();
        textData = tmp;
        while (tmp != null)
        { tmp = in.readLine();
          textData = textData + "\n" + tmp;
        }
        textData = textData.substring(0,textData.length()-5);
        loadText(textData);
        in.close();
      }
      catch (IOException ex) { System.out.println("File Not Found"); }
    }
  }

  private void loadText(String text)
  { String[] sentenceStrings = splitText(text);
    for (String s : sentenceStrings) 
    { sentence = new Sentence(s);
      if (sentence.sentenceIsCorrect() && (sentence.getSpeechAct()==Sentence.ASSERT))
      { codeString = sentence.getOwlFunctionalSyntax();
        currentWord = sentence.getCurrentWord(s.length()-1);
        dialogueLength = dialogueDoc.getLength();
        textLength = textDoc.getLength();
        updatePanesAssert(true);
      }
    }
  }

  private String[] splitText(String ts)
  { if (ts.length() == 0) ts = "$$$";
    int tsLength = ts.length();
    if (ts.charAt(tsLength-1) == '\n') ts = ts + "$$$";
    String ss[] = ts.split("\\r?\\n");
    int ssFinal = ss.length-1;
    if (ss[ssFinal].equals("$$$")) ss[ssFinal] = "";
    return ss;
  }

}
           
         