
/*

Holds explanations for inferred statements

Written by Richard Power
Copyright (c) 2013 by Open University

This package is free software; you can redistribute it and/or
modify it under the terms of the GNU Library General Public
License as published by the Free Software Foundation; either
version 3 of the License, or (at your option) any later version.

The package is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Library General Public License for more details.

==============================================================

Rule is TRANSITIVE|LACONIC|CONJUNCTION|RESTRICTION

Different rules make use of different variables:

TRANSITIVE: two statements
LACONIC: one definition
CONJUNCTION: one definition and two statements
RESTRICTION: two definitions and one statement

*/

public class Explanation
{ private Statement entailment;
  private int rule;
  private Statement statement1, statement2;
  private Definition definition1, definition2;

  public Explanation(int arg){ rule = arg; }

  public Statement getEntailment() {return entailment; }
  public int getRule() {return rule; }
  public Statement getStatement1() {return statement1; }
  public Statement getStatement2() {return statement2; }
  public Definition getDefinition1() {return definition1; }
  public Definition getDefinition2() {return definition2; }

  public void setEntailment(Statement stat) {entailment = stat; }
  public void setStatement1(Statement stat) {statement1 = stat; }
  public void setStatement2(Statement stat) {statement2 = stat; }
  public void setDefinition1(Definition def) {definition1 = def; }
  public void setDefinition2(Definition def) {definition2 = def; }

  private void displayTransitive()
  { System.out.println("SUBCLASS TRANSITIVITY RULE");
    entailment.display();
    statement1.display();
    statement2.display();
  }

  private void displayLaconic()
  { System.out.println("LACONIC RULE");
    entailment.display();
    definition1.display();
  }

  private void displayConjunction()
  { System.out.println("CONJUNCTION RULE");
    entailment.display();
    statement1.display();
    statement2.display();
    definition1.display();
  }

  private void displayRestriction()
  { System.out.println("RESTRICTION RULE");
    entailment.display();
    definition1.display();
    definition2.display();
    statement1.display();
  }

  private String reportTransitive(boolean schematic)
  { String s = "";
    s = s + entailment.reportNL() + " because:";
    if (schematic==true) s = s + " (Transitive rule)";
    s = s + "\n";
    s = s + "  (a) " + statement1.reportNL() + "\n";
    s = s + "  (b) " + statement2.reportNL();
    return s;
  }

  private String reportLaconic(boolean schematic)
  { String s = "";
    s = s + entailment.reportNL();
    if (schematic==true) s = s + " (Laconic rule)";
    return s;
  }

  private String reportConjunction(boolean schematic)
  { String s = "";
    s = s + entailment.reportNL() + " because:";
    if (schematic==true) s = s + " (Conjunction rule)";
    s = s + "\n";
    s = s + "  (a) " + statement1.reportNL() + "\n";
    s = s + "  (b) " + statement2.reportNL();
    return s;
  }

  private String reportRestriction(boolean schematic)
  { String s = "";
    s = s + entailment.reportNL() + " because:";
    if (schematic==true) s = s + " (Restriction rule)";
    s = s + "\n";
    s = s + "  (a) " + statement1.reportNL();
    return s;
  }

  public String report()
  { String s = "";
    if (rule==Model.TRANSITIVE) s = reportTransitive(true);
    if (rule==Model.LACONIC) s = reportLaconic(true);
    if (rule==Model.CONJUNCTION) s = reportConjunction(true);
    if (rule==Model.RESTRICTION) s = reportRestriction(true);
    return s;
  }

  public String reportDialogue()
  { String s = "";
    if (rule==Model.TRANSITIVE) s = reportTransitive(false);
    if (rule==Model.LACONIC) s = "Tautology.";
    if (rule==Model.CONJUNCTION) s = reportConjunction(false);
    if (rule==Model.RESTRICTION) s = reportRestriction(false);
    return s;
  }

  public void display()
  { if (rule==Model.TRANSITIVE) displayTransitive();
    if (rule==Model.LACONIC) displayLaconic();
    if (rule==Model.CONJUNCTION) displayConjunction();
    if (rule==Model.RESTRICTION) displayRestriction();
  }

}
