
/*

A glossary entry holds the text relating to a single entity (class, individual or property)

Written by Richard Power
Copyright (c) 2013 by Open University

This package is free software; you can redistribute it and/or
modify it under the terms of the GNU Library General Public
License as published by the Free Software Foundation; either
version 3 of the License, or (at your option) any later version.

The package is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Library General Public License for more details.

*/

import java.util.*;
import java.lang.String;

public class GlossaryEntry
{ private String text;
  private ModelTerm term;
  private Model model;
  private ArrayList<Statement> examples;
  private ArrayList<Statement> descriptions;

  public GlossaryEntry(ModelTerm arg1, Model arg2)
  { term = arg1;
    model = arg2;
    findExamples();
    findDescriptions();
    makeText();
  }

  public String getText() {return text; }

  public String getType()
  { String s = "INDIVIDUAL";
    if (term.getType()==Model.CLASS) s = "CLASS";
    return s;
  }

  private void findExamples()
  { examples = new ArrayList<Statement>();
    for (Statement stat : model.getStatements())
    if (stat.getPredicate().equals(term) && (tautology(stat)==false) && (stat.isLaconic()))
    examples.add(stat);
  }

  private void findDescriptions()
  { descriptions = new ArrayList<Statement>();
    for (Statement stat : model.getStatements())
    if (stat.getSubject().equals(term) && (tautology(stat)==false) & (stat.isLaconic()))
    descriptions.add(stat);
  }

  private boolean tautology(Statement stat)
  { boolean taut = false;
    for (Explanation exp : model.getExplanations())
    if (exp.getEntailment().equals(stat) && (exp.getRule()==Model.LACONIC))
    taut = true;
    return taut;
  }

  private void makeText()
  { String dtext = "";
    for (Statement d : descriptions) dtext = dtext + "   " + d.reportNL() + ".\n";
    String etext = "";
    for (Statement e : examples) etext = etext + "   " + e.reportNL() + ".\n";
    text = dtext + etext;
  }
      
}
