
/*

Tokenises an identifier of known type (individual, class, property, datatype) and
derives a suitable English verbalisation.

Written by Richard Power
Copyright (c) 2013 by Open University

This package is free software; you can redistribute it and/or
modify it under the terms of the GNU Library General Public
License as published by the Free Software Foundation; either
version 3 of the License, or (at your option) any later version.

The package is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Library General Public License for more details.

*/

import java.util.Vector;
import java.util.Collections;
import java.lang.String;

public class Identifier
{ public String englishName;
  public String fullIRI;
  public String prefix;
  public String entityName;
  public int entityType;
  public String[] words;

// Test cases
  private static String test[] =
    { "",
      "http://ontology.org/animal#fatherOf",
      "http://ontology.org/animal",
      "#cat",
      "#queenOfHearts",
      "#has_Phone_Number",
      "#_is_owned_by_",
      "lives in",
      "white van driver",
      "has as pet-owner"
    };

  public static void main(String[] args) {
    for (int i=0; i<test.length; i++)
    { Identifier id = new Identifier(OwlTerm.OBJECT_PROPERTY,test[i]);
      id.display();
    }
  }

  public Identifier(int arg1,String arg2)
  { entityType = arg1;
    fullIRI = arg2;
    parseIRI();
    englishName = splitByUnderlineAndCamelCase(entityName);
    if ((entityType == OwlTerm.OBJECT_PROPERTY) ||
        (entityType == OwlTerm.DATA_PROPERTY))
    revisePropertyName();
    else if (entityType == OwlTerm.CLASS) reviseClassName();
    else if (entityType == OwlTerm.NAMED_INDIVIDUAL) reviseIndividualName();
    else if (entityType == OwlTerm.DATATYPE) reviseDatatype();
  }

  public String getEnglishName() {return englishName; }
  public String getPrefix() {return prefix; }
  public String getEntityName() {return entityName; }

// Separate prefix from entity name
  private void parseIRI()
  { prefix = "";
    entityName = fullIRI;
    int iL = fullIRI.length();
    int pL = iL;
    while ((pL > 1) && (!prefixEnding(fullIRI.charAt(pL-1)))) pL = pL-1;
    if ((iL > 0) && (prefixEnding(fullIRI.charAt(pL-1))))
    { prefix = fullIRI.substring(0,pL);
      entityName = fullIRI.substring(pL,iL);
    }
  }

 private boolean prefixEnding(char c)
  { boolean tval = false;
    if (c == '#') tval = true;
    if (c == '/') tval = true;
    if (c == ':') tval = true;
    return tval;
  }

 private String splitByUnderlineAndCamelCase(String s)
  { s = s.replace(" ","_");
    String result = s.replaceAll(
      String.format("%s|%s|%s",
        "(?<=[A-Z])(?=[A-Z][a-z])",
        "(?<=[^A-Z])(?=[A-Z])",
        "(?<=[A-Za-z])(?=[^A-Za-z])"
      ),
      " "
    );
    result = result.replace("_","");
    result = result.replace("  "," ");
    result = result.replace(" -","-");
    return result;
  }

// Find out whether first word ends in -s, otherwise add "is" at front
// Also ensure lower-case throughout
  private void revisePropertyName()
  { int i = 0;
    if (englishName.length() > 0)
    { while ((i < englishName.length()-1) && (englishName.charAt(i+1) != ' ')) i++;
      if (englishName.charAt(i) != 's') englishName = "is " + englishName;
    }
    englishName = englishName.toLowerCase();
  }

// As a rough guess, ensure lower-case throughout
  private void reviseClassName()
  { englishName = englishName.toLowerCase();
  }

// Capitalize first word
  private void reviseIndividualName()
  { englishName = openingCapital(englishName);
  }

// Convert integer to integer-values etc. for datatypes
  private void reviseDatatype()
  { englishName = englishName + "-value";
  }

  private String openingCapital(String s)
  { String cs = s.substring(0,1).toUpperCase()+ s.substring(1,s.length());
    return cs;
  }

  public void display()
  { System.out.println(fullIRI);
    System.out.println(prefix + " " + entityName);
    System.out.println(englishName + "\n");
  }
    
}
