
/*

Holds subsumption statements for inference model

Written by Richard Power
Copyright (c) 2013 by Open University

This package is free software; you can redistribute it and/or
modify it under the terms of the GNU Library General Public
License as published by the Free Software Foundation; either
version 3 of the License, or (at your option) any later version.

The package is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Library General Public License for more details.

*/

public class Statement
{ private ModelTerm subject;
  private ModelTerm predicate;
  private int source;
  private int distance;
  private boolean laconic; // false if predicate is a conjunction

  public Statement(ModelTerm arg1, ModelTerm arg2, int arg3, int arg4)
  { subject = arg1;
    predicate = arg2;
    source = arg3;
    distance = arg4;
  }

  public ModelTerm getSubject() { return subject; }
  public ModelTerm getPredicate() { return predicate; }
  public int getSource() { return source; }
  public int getDistance() { return distance; }
  public boolean isLaconic() { return laconic; }

  public void setSubject(ModelTerm mterm) { subject = mterm; }
  public void setPredicate(ModelTerm mterm) { predicate = mterm; }
  public void setSource(int arg) {source = arg; }
  public void setDistance(int arg) {distance = arg; }
  public void setLaconic(boolean arg) {laconic = arg; }

  public String report()
  { String s = subject.getName();
    s = s + " => " + predicate.getName();
    if (source == Model.ORIGINAL) s = s + " (*)";
    return s;
  }

  public String reportNL()
  { String s = getPhrase();
    return openingCapital(s);
  }

  public String reportPhrase()
  { return getPhrase() + ".";
  }

  private String getPhrase()
  { String s = "";
    if (subject.getType()==Model.INDIVIDUAL)
    s = s + subject.getName();
    else s = "every " + subject.getReportName();
    if (predicate.getType()==Model.INDIVIDUAL)
    s = s + " is " + subject.getName();
    else if (predicate.getCategory()==Model.VERB_PHRASE)
    s = s + " " + predicate.getName();
    else s = s + " is " + indefiniteArticle(predicate.getName()) + " " + predicate.getName();
    return s;
  }

// Make sure the opening letter is upper-case at start of sentence
  private String openingCapital(String s)
  { String cs = s.substring(0,1).toUpperCase()+ s.substring(1,s.length());
    return cs;
  }

  private String indefiniteArticle(String noun)
  { String indef = "a";
    if ((noun.charAt(0)=='a') ||
        (noun.charAt(0)=='e') ||
        (noun.charAt(0)=='i') ||
        (noun.charAt(0)=='o') ||
        (noun.charAt(0)=='u') ||
        (noun.charAt(0)=='A') ||
        (noun.charAt(0)=='E') ||
        (noun.charAt(0)=='I') ||
        (noun.charAt(0)=='O') ||
        (noun.charAt(0)=='U'))
    indef = "an";
    return indef;
  }

  public void display()
  { subject.display();
    System.out.print(" ");
    predicate.display();
    if (source == Model.ORIGINAL) System.out.print(" (*)");
    System.out.println();
  }

}
